@@ -14,7 +14,9 @@
 
 #undef USE_NCURSES_H
 
-#undef CURSES_BOOL
+#undef LE_CURSES_BOOL_TYPE
+
+#undef DISABLE_FILE_LOCKS
 
 @BOTTOM@
 
