@@ -17,7 +17,7 @@
  */
 
 /* block.cc : block operations */
-/* $Id: block.cc,v 1.16 1999/10/31 12:12:39 lav Exp $ */
+/* $Id: block.cc,v 1.22 2001/04/09 08:24:56 lav Exp $ */
 
 #include <config.h>
 #include <assert.h>
@@ -71,6 +71,8 @@ void   MoveLineCol(num l,num c)
 }
 void   HideDisplay()
 {
+   if(DragMark)
+      UserStopDragMark();
    hide=!hide;
    CheckBlock();
    flag=1;
@@ -120,7 +122,7 @@ void   ExpandTab()
 
 void    RCopy()
 {
-   Message("Copying...");
+   MessageSync("Copying...");
 
    ClipBoard cb;
    TextPoint tp=CurrentPos;
@@ -138,6 +140,9 @@ void    RCopy()
 
 void    Copy()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    CheckBlock();
    if(View || hide)
       return;
@@ -174,7 +179,7 @@ static int lines_deleted;
 
 int   RDelete()
 {
-   Message("Deleting...");
+   MessageSync("Deleting...");
 
    num   i,j;
    num   h=BlockEnd.Line()-BlockBegin.Line()+1;
@@ -242,6 +247,9 @@ int   RDelete()
 }
 int   Delete()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    CheckBlock();
    if(View || hide)
       return 1;
@@ -294,6 +302,9 @@ void    RMove()
 }
 void    Move()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    num	 size=BlockEnd-BlockBegin;
 
    CheckBlock();
@@ -330,6 +341,9 @@ void    Move()
 
 void    Write()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    int         fd;
    struct stat st;
    num         act_written;
@@ -345,7 +359,7 @@ void    Write()
    {
       LoadHistory.Push();
       /* write to pipe */
-      Message("Piping to...");
+      MessageSync("Piping to...");
       PipeBlock(BlockFile+1,/*IN*/FALSE,/*OUT*/TRUE);
       return;
    }
@@ -410,7 +424,7 @@ void    Write()
       return;
    }
    errno=0;
-   Message("Writing...");
+   MessageSync("Writing...");
    if(rblock)
    {
       ClipBoard cb;
@@ -486,6 +500,9 @@ int OptionallyConvertBlockNewLines(const char *bname)
 
 void    Read()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    int             fd;
    struct stat     st;
    num   act_read;
@@ -500,7 +517,7 @@ void    Read()
    {
       LoadHistory.Push();
       /* read from pipe */
-      Message("Piping in...");
+      MessageSync("Piping in...");
       if(PipeBlock(BlockFile+1,TRUE,FALSE)==OK)
 	 goto after_read;
       return;
@@ -527,7 +544,7 @@ void    Read()
        close(fd);
        return;
    }
-   Message("Reading...");
+   MessageSync("Reading...");
    PreUserEdit();
    if(buffer_mmapped || (hex && !insert))
       res=ReadBlockOver(fd,st.st_size,&act_read);
@@ -545,10 +562,10 @@ void    Read()
       return;
    BlockBegin=BlockEnd=CurrentPos;
    BlockBegin-=act_read;
-   flag=REDISPLAY_ALL;
    rblock=hide=FALSE;
 
 after_read:
+   flag=REDISPLAY_ALL;
    OptionallyConvertBlockNewLines("read");
    CurrentPos=BlockEnd;
    stdcol=GetCol();
@@ -599,8 +616,9 @@ void    DoIndent(int i)
             InsertChar('\t');
       for(; j>0; j--)
          InsertChar(' ');
-      MoveDown();
-      ToLineBegin();
+      MoveRight();
+      while(!Bol() && !Eof())
+	 MoveRight();
    }
    while(CurrentPos<BlockEnd);
    BlockEnd=CurrentPos;
@@ -651,8 +669,9 @@ void    DoUnindent(int i)
             InsertChar('\t');
       for(; j>0; j--)
          InsertChar(' ');
-      MoveDown();
-      ToLineBegin();
+      MoveRight();
+      while(!Bol() && !Eof())
+	 MoveRight();
    }
    while(CurrentPos<BlockEnd);
    BlockEnd=CurrentPos;
@@ -663,6 +682,9 @@ void    DoUnindent(int i)
 char   is[64]="";
 void   Indent()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    int    i;
    CheckBlock();
    if(View || hide)
@@ -683,6 +705,9 @@ void   Indent()
 }
 void   Unindent()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    int    i;
    CheckBlock();
    if(View || hide)
@@ -757,6 +782,7 @@ next:
       Message("Block: B-Begin E-End T-Type H-display A-mark All");
    else
       Message("Block: W-Write B-Begin E-End T-Type H-Hide A-mark All");
+   SyncTextWin();
    SetCursor();
    action=GetNextAction();
    flag=REDISPLAY_ALL;
@@ -905,20 +931,29 @@ void  Transform(byte (*func)(byte))
 
 void    ConvertToUpper()
 {
-    Transform(Toupper);
+   if(DragMark)
+      UserStopDragMark();
+
+   Transform(Toupper);
 }
 void    ConvertToLower()
 {
-    Transform(Tolower);
+   if(DragMark)
+      UserStopDragMark();
+
+   Transform(Tolower);
 }
 void    ExchangeCases()
 {
-    Transform(Inverse);
+   if(DragMark)
+      UserStopDragMark();
+
+   Transform(Inverse);
 }
 void    BlockType()
 {
-    rblock=!rblock;
-    flag=!hide;
+   rblock=!rblock;
+   flag=!hide;
 }
 
 void    CheckBlock()
@@ -938,6 +973,9 @@ void    CheckBlock()
 
 void  PrefixIndent(char *prefix,num len)
 {
+   if(DragMark)
+      UserStopDragMark();
+
    CheckBlock();
    if(hide)
       return;
