@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: clipbrd.cc,v 1.5 1999/07/11 10:48:11 lav Exp $ */
+/* $Id: clipbrd.cc,v 1.6 2002/08/06 13:05:34 lav Exp $ */
 
 #include <config.h>
 #include <stdlib.h>
@@ -76,7 +76,7 @@ int ClipBoard::Copy()
       }
       width=col2-col1;
       height=line2-line1+1;
-      text=(char*)malloc(width*height);
+      text=(char*)malloc(width*height+1);
       if(!text)
       {
 	 NotMemory();
@@ -268,7 +268,7 @@ int ClipBoard::Linearize(char **buf,int *len)
 {
    if(!text)
       return 0;
-   *buf=(char*)malloc(width*height);
+   *buf=(char*)malloc((width+EolSize)*height+1);
    if(!*buf)
    {
       NotMemory();
