@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: edit.h,v 1.16 1999/11/06 16:57:54 lav Exp $ */
+/* $Id: edit.h,v 1.22 2001/04/13 08:56:01 lav Exp $ */
 
 #ifndef EDIT_H
 #define EDIT_H
@@ -24,8 +24,8 @@
 #include    <sys/types.h>
 #include    <time.h>
 
-#if defined(CURSES_BOOL) && !defined(bool)
-# define bool CURSES_BOOL
+#if !defined(bool)
+# define bool LE_CURSES_BOOL_TYPE
 # define bool_redefined 1
 #endif
 #ifdef USE_NCURSES_H
@@ -58,7 +58,6 @@ typedef long            num;
 #include "menu1.h"
 #include "cmd.h"
 #include "history.h"
-#include "search.h"
 #include "file.h"
 
 extern  int inputmode,editmode,noreg;
@@ -227,6 +226,8 @@ void    GoToOffset(void);
 void    GoToLineNumber(void);
 void    GoToLineNum(num);
 
+void  SeekStdCol();
+
 void  CheckWindowResize();
 
 extern  void    Quit(void),
@@ -239,7 +240,7 @@ extern  void    Quit(void),
                 ExpandAllTabs(void),
                 TermOpt(void),SaveOpt(void),UpdtOpt(void),AppearOpt(void),
                 edit_chset(void),SaveTermOpt(void),GoToLineNumber(void),
-                FormatOptions(void),Optimize(void),DOS_UNIX(void);
+                FormatOptions(void),DOS_UNIX(void);
 
 void  PreModify();
 int   PreUserEdit();
@@ -252,11 +253,13 @@ int   CopyBlock(offs from,num len);
 int   CopyBlockOver(offs from,num len);
 int   ReadBlock(int fd,num len,num *act_read);
 int   ReadBlockOver(int fd,num len,num *act_read);
+int   ReplaceTextFromFile(int fd,num len,num *act_read);
 int   WriteBlock(int fd,offs from,num len,num *act_written);
 int   DeleteBlock(num left,num right);
 int   GetBlock(char *copy,offs from,num size);
 int   Undelete();
 void  CheckPoint();
+offs  ScanForCharForward(offs start,byte ch);
 
 void  Help(const char *help,const char *title);
 /*void  Help(char ***help,char *title);*/
