@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1993-2000 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -63,7 +63,7 @@ int   getstring(const char *pr,char *buf,int maxlen,History* history,int *len,
       if(pos-shift<0)
          shift=pos;
       SetAttr(STATUS_LINE_ATTR);
-      mvaddstr(LINES-1,0,pr);
+      mvaddstr(LINES-1,0,(char*)pr);
       for(i=0; i<width && i+shift<(*len); i++)
       {
          addch_visual((byte)buf[i+shift]);
