@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: kern.cc,v 1.12 1999/08/25 23:30:27 lav Exp $ */
+/* $Id: kern.cc,v 1.16 2001/04/13 08:56:01 lav Exp $ */
 
 #include <config.h>
 #include   <stdio.h>
@@ -32,6 +32,7 @@
 #include   <unistd.h>
 #include   <time.h>
 #include   <stdlib.h>
+#include   <assert.h>
 #include   "keymap.h"
 #include   "edit.h"
 
@@ -164,9 +165,12 @@ void   ToLineEnd()
    todelete-=ptr;
    CurrentPos=ptr;
    om=modified;
-   DeleteBlock(0,todelete);
    if(todelete>0)
+   {
+      DeleteBlock(0,todelete);
       oldptr2=ptr2;
+      flag|=REDISPLAY_LINE;
+   }
    modified=om;
    stdcol=GetCol();
 }
@@ -510,6 +514,37 @@ int   ReadBlock(int fd,num size,num *act_read)
    return(InsertBlock(buffer+ptr1,*act_read));
 }
 
+int   ReplaceTextFromFile(int fd,num size,num *act_read)
+{
+   // buffer should be clear
+   assert(Size()==0);
+
+   if(buffer_mmapped)
+      return ERR;
+
+   if(size==0)
+   {
+      *act_read=0;
+      return(OK);
+   }
+
+   if(GetSpace(size)!=OK)
+      return(ERR);
+
+   *act_read=read(fd,buffer+ptr1,size);
+   if(*act_read==-1)
+      return(ERR);
+   if(*act_read==0)
+      return(OK);
+
+   ptr1+=*act_read;
+   GapSize-=*act_read;
+
+   TextEnd=TextPoint(Size());
+
+   return(OK);
+}
+
 int   ReadBlockOver(int fd,num size,num *act_read)
 {
    if(buffer_mmapped)
@@ -970,35 +1005,6 @@ void  EmptyText()
    EolStr="\n";
 }
 
-void   Optimize()
-{
-   if(View || buffer_mmapped)
-      return;
-
-   offs     ptr;
-   TextPoint  tp=CurrentPos;
-
-   Message("Optimizing...");
-   for(ptr=0; !EofAt(ptr); ptr++)
-   {
-      if(EolAt(ptr))
-      {
-         CurrentPos=ptr;
-         while(!Bol() && (CharRel_NoCheck(-1)==' ' || CharRel_NoCheck(-1)=='\t'))
-            BackSpace();
-      }
-   }
-   CurrentPos=TextEnd;
-   while(!Bof() && Bol() && BolAt(Offset()-EolSize))
-      DeleteBlock(EolSize,0);
-   if(!Bol())
-      NewLine();
-
-   CurrentPos=tp;
-   stdcol=GetCol();
-   flag=REDISPLAY_ALL;
-}
-
 char  *GetWord()
 {
    static   char  word[256];
@@ -1077,3 +1083,34 @@ void  ConvertFromUnixToDos(offs start,num size)
       start++;
    }
 }
+
+void  SeekStdCol()
+{
+   num std=stdcol;
+   if(!hex)
+   {
+      while(GetCol()>std)
+	MoveLeft();
+      while(GetCol()<std && !Eol())
+	MoveRight();
+   }
+}
+
+offs  ScanForCharForward(offs start,byte ch)
+{
+   char *pos;
+   if(start<ptr1)
+   {
+      pos=(char*)memchr(buffer+start,ch,ptr1-start);
+      if(pos)
+	 return pos-buffer;
+      start=ptr1;
+   }
+   if(start<Size())
+   {
+      pos=(char*)memchr(buffer+GapSize+start,ch,Size()-start);
+      if(pos)
+	 return pos-buffer-GapSize;
+   }
+   return -1;
+}
