@@ -30,6 +30,7 @@
 #include "keymap.h"
 #include "keynames.h"
 #include "getch.h"
+#include <term.h>
 
 char  StringTyped[256];
 int   StringTypedLen;
@@ -94,6 +95,7 @@ ActionNameRec  ActionNameTable[]=
 // File ops
    {LOAD_FILE,"load-file"},
    {SWITCH_FILE,"switch-file"},
+   {REOPEN_FILE_RW,"reopen-file-rw"},
    {SAVE_FILE,"save-file"},
    {SAVE_FILE_AS,"save-file-as"},
    {FILE_INFO,"file-info"},
@@ -118,8 +120,10 @@ ActionNameRec  ActionNameTable[]=
    {BLOCK_FUNC_BAR,"block-functions"},
    {MARK_LINE,"mark-line"},
    {MARK_TO_EOL,"mark-to-eol"},
+   {MARK_ALL,"mark-all"},
    {START_DRAG_MARK,"start-drag-mark"},
    {YANK_BLOCK,"yank-block"},
+   {REMEMBER_BLOCK,"remember-block"},
 
 // Search
    {SEARCH_FORWARD,"search-forward"},
@@ -134,7 +138,7 @@ ActionNameRec  ActionNameTable[]=
    {FORMAT_ONE_PARA,"format-paragraph"},
    {FORMAT_DOCUMENT,"format-document"},
    {CENTER_LINE,"center-line"},
-   {AJUST_RIGHT_LINE,"ajust-right-line"},
+   {ADJUST_RIGHT_LINE,"adjust-right-line"},
    {FORMAT_FUNC_BAR,"format-functions"},
 
 // Others
@@ -163,6 +167,7 @@ ActionNameRec  ActionNameTable[]=
 
    {SUSPEND_EDITOR,"suspend-editor"},
    {QUIT_EDITOR,"escape"},
+   {QUIT_EDITOR,"quit-editor"},
 
    {COMPILE_CMD,"compile"},
    {MAKE_CMD,"make"},
@@ -183,6 +188,49 @@ ActionNameRec  ActionNameTable[]=
    {SWITCH_TEXT_MODE,"switch-text-mode"},
    {SWITCH_GRAPH_MODE,"switch-graph-mode"},
 
+   {EDIT_CHARSET,"edit-charset"},
+   {SET_CHARSET_8BIT,"set-charset-8bit"},
+   {SET_CHARSET_8BIT_NO_CONTROL,"set-charset-8bit-no-control"},
+   {SAVE_TERMINAL_OPTIONS,"save-terminal-options"},
+   {EDIT_COLORS,"edit-colors"},
+   {SAVE_COLORS,"save-colors"},
+   {SAVE_COLORS_FOR_TERM,"save-colors-for-terminal"},
+   {LOAD_COLOR_DEFAULT,"load-color-default"},
+   {LOAD_COLOR_DEFBG,"load-color-defbg"},
+   {LOAD_COLOR_BLACK,"load-color-black"},
+   {LOAD_COLOR_BLUE,"load-color-blue"},
+   {LOAD_COLOR_GREEN,"load-color-green"},
+   {LOAD_COLOR_WHITE,"load-color-white"},
+   {PROGRAMS_OPTIONS,"programs-options"},
+   {ABOUT,"about"},
+   {LOAD_KEYMAP_DEFAULT,"load-keymap-default"},
+   {LOAD_KEYMAP_EMACS,  "load-keymap-emacs"},
+   {SAVE_KEYMAP,"save-keymap"},
+   {SAVE_KEYMAP_FOR_TERM,"save-keymap-for-terminal"},
+
+   {SET_BOOKMARK,"set-bookmark"},
+   {SET_BOOKMARK_0,"set-bookmark-0"},
+   {SET_BOOKMARK_1,"set-bookmark-1"},
+   {SET_BOOKMARK_2,"set-bookmark-2"},
+   {SET_BOOKMARK_3,"set-bookmark-3"},
+   {SET_BOOKMARK_4,"set-bookmark-4"},
+   {SET_BOOKMARK_5,"set-bookmark-5"},
+   {SET_BOOKMARK_6,"set-bookmark-6"},
+   {SET_BOOKMARK_7,"set-bookmark-7"},
+   {SET_BOOKMARK_8,"set-bookmark-8"},
+   {SET_BOOKMARK_9,"set-bookmark-9"},
+   {GO_BOOKMARK,"go-bookmark"},
+   {GO_BOOKMARK_0,"go-bookmark-0"},
+   {GO_BOOKMARK_1,"go-bookmark-1"},
+   {GO_BOOKMARK_2,"go-bookmark-2"},
+   {GO_BOOKMARK_3,"go-bookmark-3"},
+   {GO_BOOKMARK_4,"go-bookmark-4"},
+   {GO_BOOKMARK_5,"go-bookmark-5"},
+   {GO_BOOKMARK_6,"go-bookmark-6"},
+   {GO_BOOKMARK_7,"go-bookmark-7"},
+   {GO_BOOKMARK_8,"go-bookmark-8"},
+   {GO_BOOKMARK_9,"go-bookmark-9"},
+
    {-1,NULL}
 };
 
@@ -209,6 +257,7 @@ struct KeyTreeNode
 
 extern   ActionCodeRec  DefaultActionCodeTable[];
 ActionCodeRec  *ActionCodeTable=DefaultActionCodeTable;
+bool NeedFreeActionCodeTable=false;
 //char  *ti_cache[128]={NULL};
 
 char  *GetActionName(int action)
@@ -219,7 +268,7 @@ char  *GetActionName(int action)
    return(NULL);
 }
 
-char  *GetActionCodeText(char *code)
+const char *GetActionCodeText(const char *code)
 {
    static char code_text[1024];
    char  *store=code_text;
@@ -244,11 +293,190 @@ char  *GetActionCodeText(char *code)
    return(code_text);
 }
 
+#define LEFT_BRACE  '{'
+#define RIGHT_BRACE '}'
+
+static int PrettyCodeScore(const char *c)
+{
+   if(c==0)
+      return 1000000;
+
+   int score=0;
+   while(*c)
+   {
+      score++;
+
+      char  term_name[256];
+      char  *term_str;
+      int   bracket;
+      int   fk;
+      int   shift;
+      char code_ch=*c;
+      switch(code_ch)
+      {
+      case('$'):
+	 code_ch=*(++c);
+
+	 if(code_ch==0)
+	    break;
+
+	 bracket=(code_ch==LEFT_BRACE);
+	 c+=bracket;
+
+	 term_str=term_name;
+	 while(*c!=0 && (bracket?*c!=RIGHT_BRACE:isalnum(*c)) && term_str-term_name<255)
+	    *term_str++=*c++;
+	 *term_str=0;
+	 if(!(bracket && *c==RIGHT_BRACE))
+	    c--;
+
+	 if(sscanf(term_name,"%1dkf%d",&shift,&fk)==2
+	 || sscanf(term_name,"kf%d",&fk)==1)
+	 {
+	    if(shift)
+	       score+=4*shift;
+	    else
+	       score+=2;
+	 }
+	 else
+	    score+=8;
+
+	 break;
+      case('|'):
+	 score+=5;
+	 break;
+      case('^'):;
+      case('\\'):;
+      }
+      c++;
+   }
+   return score;
+}
+
+const char *ActionCodePrettyPrint(const char *c)
+{
+   static char code_text[1024];
+   char  *store=code_text;
+   *store=0;
+
+   while(*c)
+   {
+      char  term_name[256];
+      char  *term_str;
+      int   bracket;
+      int   fk;
+      int   shift;
+      unsigned char code_ch=*c;
+      switch(code_ch)
+      {
+      case('$'):
+	 code_ch=*(++c);
+
+	 if(code_ch==0)
+	    break;
+
+	 bracket=(code_ch==LEFT_BRACE);
+	 c+=bracket;
+
+	 term_str=term_name;
+	 while(*c!=0 && (bracket?*c!=RIGHT_BRACE:isalnum(*c)) && term_str-term_name<255)
+	    *term_str++=*c++;
+	 *term_str=0;
+	 if(!(bracket && *c==RIGHT_BRACE))
+	    c--;
+
+	 shift=0;
+	 if((sscanf(term_name,"%1dkf%d",&shift,&fk)==2
+	  || sscanf(term_name,"kf%d",&fk)==1) && shift<4)
+	 {
+	    static char shift_str_map[][3]={"","~","^","~^"};
+	    store+=sprintf(store,"%sF%d",shift_str_map[shift],fk);
+	 }
+	 else
+	 {
+	    // FIXME.
+	    store+=sprintf(store,"%s",term_name);
+	 }
+	 if(c[1] && c[1]!='|')
+	 {
+	    *store++=' ';
+	    *store=0;
+	 }
+	 break;
+      case('|'):
+	 *store++='+';
+	 *store=0;
+	 break;
+      case('^'):
+	 if(c[1])
+	 {
+	    *store++='^';
+	    *store++=toupper(*++c);
+	    *store=0;
+	    break;
+	 }
+	 goto default_l;
+      case('\\'):
+	 code_ch=*(++c);
+      default:
+      default_l:
+	 if(code_ch==27 && c[1]=='|' && c[2] && c[2]!='$')
+	 {
+	    *store++='M';
+	    *store++='-';
+	    *store=0;
+	    c++;
+	 }
+	 else if(code_ch<32)
+	 {
+	    *store++='^';
+	    *store++=code_ch+'@';
+	 }
+	 else if(code_ch==128)
+	 {
+	    *store++='^';
+	    *store++='@';
+	 }
+	 else
+	    *store++=code_ch;
+	 *store=0;
+      }
+      c++;
+   }
+   return code_text;
+}
+
+const char *ShortcutPrettyPrint(int c)
+{
+   static char code_text[1024];
+   char  *store=code_text;
+
+   const char *best_code=0;
+   int best_score=1000000;
+   for(int i=0; ActionCodeTable[i].action!=-1; i++)
+   {
+      if(ActionCodeTable[i].action!=c)
+	 continue;
+      const char *code=ActionCodeTable[i].code;
+      int score=PrettyCodeScore(code);
+      if(score<best_score)
+      {
+	 best_code=code;
+	 best_score=score;
+      }
+   }
+   if(best_code==0)
+      return 0;
+
+   strcpy(store,ActionCodePrettyPrint(best_code));
+   return code_text;
+}
+
 void  WriteActionMap(FILE *f)
 {
    for(int i=0; ActionCodeTable[i].action!=-1; i++)
    {
-      fprintf(f,"%-20s %s\n",GetActionName(ActionCodeTable[i].action),
+      fprintf(f,"%-23s %s\n",GetActionName(ActionCodeTable[i].action),
                              GetActionCodeText(ActionCodeTable[i].code));
    }
 }
@@ -333,7 +561,7 @@ KeyTreeNode *BuildKeyTree(ActionCodeRec *ac_table)
 	       code+=bracket;
 
 	       term_str=term_name;
-	       while(*code!=0 && (bracket?*code==RIGHT_BRACE:isalnum(*code)) && term_str-term_name<255)
+	       while(*code!=0 && (bracket?*code!=RIGHT_BRACE:isalnum(*code)) && term_str-term_name<255)
 		  *term_str++=*code++;
 	       *term_str=0;
 	       if(bracket && *code==RIGHT_BRACE)
@@ -358,7 +586,7 @@ KeyTreeNode *BuildKeyTree(ActionCodeRec *ac_table)
 		     delay=HALF_DELAY;
 		     term_str++;
 		  }
-		  key_code=term_str[0];
+		  key_code=(unsigned char)term_str[0];
 		  if(key_code==0)
 		     goto fallback;
 	       }
@@ -381,7 +609,7 @@ KeyTreeNode *BuildKeyTree(ActionCodeRec *ac_table)
 	       code_ch=*(++code);
 	    default:
 	    default_l:
-	       key_code=code_ch;
+	       key_code=(unsigned char)code_ch;
 	       code++;
 	    }
 
@@ -417,8 +645,22 @@ void RebuildKeyTree()
    KeyTree=BuildKeyTree(ActionCodeTable);
 }
 
+int FindActionCode(const char *ActionName)
+{
+   int action_no;
+
+   /* find the named action in table */
+   for(action_no=0; ActionNameTable[action_no].action!=-1
+      && strcmp(ActionNameTable[action_no].name,ActionName); action_no++);
+
+   return ActionNameTable[action_no].action;
+}
+
+
 void  ReadActionMap(FILE *f)
 {
+   FreeActionCodeTable();
+
    char  ActionName[256];
    char  ActionCode[256];
    char  *store;
@@ -426,7 +668,6 @@ void  ReadActionMap(FILE *f)
    ActionCodeRec  *NewTable=NULL;
    int   CurrTableSize=0;
    int   CurrTableCell=0;
-   int   action_no;
 
    for(;;)  /* line cycle */
    {
@@ -441,15 +682,13 @@ void  ReadActionMap(FILE *f)
       }
       *store=0;
 
-      /* find the named action in table */
-      for(action_no=0; ActionNameTable[action_no].action!=-1
-         && strcmp(ActionNameTable[action_no].name,ActionName); action_no++);
-
-      /* is the action found in the table ? */
-      if(ActionNameTable[action_no].action==-1)
+      int action_found=FindActionCode(ActionName);
+      if(action_found==-1)
       {
          while(ch!='\n' && ch!=EOF)
             ch=fgetc(f);
+	 if(ch==EOF)
+	    break;
 	 continue;
       }
 
@@ -497,7 +736,7 @@ void  ReadActionMap(FILE *f)
             }
          }
          if(ch=='\000')
-            ch='\200';
+            ch=128;
 
          if(store-ActionCode<(int)sizeof(ActionCode)-1)
             *(store++)=ch;
@@ -520,7 +759,7 @@ void  ReadActionMap(FILE *f)
             exit(1);
          }
       }
-      NewTable[CurrTableCell].action=ActionNameTable[action_no].action;
+      NewTable[CurrTableCell].action=action_found;
       NewTable[CurrTableCell].code=strdup(ActionCode);
       if(NewTable[CurrTableCell].code==NULL)
       {
@@ -540,6 +779,21 @@ void  ReadActionMap(FILE *f)
    NewTable[CurrTableCell].code=NULL;
 
    ActionCodeTable=NewTable;
+   NeedFreeActionCodeTable=true;
+}
+
+void FreeActionCodeTable()
+{
+   if(!NeedFreeActionCodeTable)
+   {
+      ActionCodeTable=0;
+      return;
+   }
+   for(int i=0; ActionCodeTable[i].code; i++)
+      free(ActionCodeTable[i].code);
+   free(ActionCodeTable);
+   ActionCodeTable=0;
+   NeedFreeActionCodeTable=false;
 }
 
 int   GetNextAction()
@@ -621,7 +875,7 @@ int   GetNextAction()
 	       break;
 	 if(!scan)
 	 {
-	    if(StringTypedLen>1) {
+	    if(StringTypedLen>1 && kt->action==NO_ACTION) {
 	    // We've got an unknown sequence.
 	    // It is likely that it is a bit longer that we've already got,
 	    // so try to flush it.
