@@ -77,6 +77,7 @@ enum  Action
 // File ops
    LOAD_FILE,
    SWITCH_FILE,
+   REOPEN_FILE_RW,
    SAVE_FILE,
    SAVE_FILE_AS,
    FILE_INFO,
@@ -101,8 +102,10 @@ enum  Action
    BLOCK_FUNC_BAR,
    MARK_LINE,
    MARK_TO_EOL,
+   MARK_ALL,
    START_DRAG_MARK,
    YANK_BLOCK,
+   REMEMBER_BLOCK,
 
 // Search
    SEARCH_FORWARD,
@@ -117,7 +120,7 @@ enum  Action
    FORMAT_ONE_PARA,
    FORMAT_DOCUMENT,
    CENTER_LINE,
-   AJUST_RIGHT_LINE,
+   ADJUST_RIGHT_LINE,
    FORMAT_FUNC_BAR,
 
 // Others
@@ -168,6 +171,49 @@ enum  Action
    SWITCH_TEXT_MODE,
    SWITCH_GRAPH_MODE,
 
+   EDIT_CHARSET,
+   SET_CHARSET_8BIT,
+   SET_CHARSET_8BIT_NO_CONTROL,
+   SAVE_TERMINAL_OPTIONS,
+   EDIT_COLORS,
+   SAVE_COLORS,
+   SAVE_COLORS_FOR_TERM,
+   LOAD_COLOR_DEFAULT,
+   LOAD_COLOR_DEFBG,
+   LOAD_COLOR_BLACK,
+   LOAD_COLOR_BLUE,
+   LOAD_COLOR_GREEN,
+   LOAD_COLOR_WHITE,
+   PROGRAMS_OPTIONS,
+   ABOUT,
+   LOAD_KEYMAP_DEFAULT,
+   LOAD_KEYMAP_EMACS,
+   SAVE_KEYMAP,
+   SAVE_KEYMAP_FOR_TERM,
+
+   SET_BOOKMARK,
+   SET_BOOKMARK_0,
+   SET_BOOKMARK_1,
+   SET_BOOKMARK_2,
+   SET_BOOKMARK_3,
+   SET_BOOKMARK_4,
+   SET_BOOKMARK_5,
+   SET_BOOKMARK_6,
+   SET_BOOKMARK_7,
+   SET_BOOKMARK_8,
+   SET_BOOKMARK_9,
+   GO_BOOKMARK,
+   GO_BOOKMARK_0,
+   GO_BOOKMARK_1,
+   GO_BOOKMARK_2,
+   GO_BOOKMARK_3,
+   GO_BOOKMARK_4,
+   GO_BOOKMARK_5,
+   GO_BOOKMARK_6,
+   GO_BOOKMARK_7,
+   GO_BOOKMARK_8,
+   GO_BOOKMARK_9,
+
    MOUSE_ACTION,
    NO_ACTION
 };
@@ -197,6 +243,7 @@ extern   int   StringTypedLen;
 extern   ActionProcRec  EditorActionProcTable[];
 extern   ActionCodeRec  *ActionCodeTable;
 extern   ActionCodeRec  DefaultActionCodeTable[];
+extern   bool NeedFreeActionCodeTable;
 
 int   GetNextAction(void);
 char  *GetActionString(int action);
@@ -205,5 +252,15 @@ void  WriteActionMap(FILE*);
 ActionProc  GetActionProc(ActionProcRec*,int action);
 void  EditorReadKeymap();
 void  RebuildKeyTree();
+void  FreeActionCodeTable();
+
+int   FindActionCode(const char *);
+
+const char *ShortcutPrettyPrint(int c);
+
+void LoadKeymapEmacs();
+void LoadKeymapDefault();
+void SaveKeymap();
+void SaveKeymapForTerminal();
 
 #endif /* KEYMAP_H */
