@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1993-2000 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@
 #include "options.h"
 #include "keymap.h"
 #include "format.h"
+#include "search.h"
+#include "colormnu.h"
 
 ActionProcRec  EditorActionProcTable[]=
 {
@@ -82,6 +84,7 @@ ActionProcRec  EditorActionProcTable[]=
 // File ops
    {LOAD_FILE,UserLoad},
    {SWITCH_FILE,UserSwitch},
+   {REOPEN_FILE_RW,(void(*)())ReopenRW},
    {SAVE_FILE,(void(*)())UserSave},
    {SAVE_FILE_AS,(void(*)())UserSaveAs},
    {FILE_INFO,UserInfo},
@@ -106,8 +109,10 @@ ActionProcRec  EditorActionProcTable[]=
    {BLOCK_FUNC_BAR,BlockFunc},
    {MARK_LINE,UserMarkLine},
    {MARK_TO_EOL,UserMarkToEol},
+   {MARK_ALL,UserMarkAll},
    {START_DRAG_MARK,UserStartDragMark},
    {YANK_BLOCK,UserYankBlock},
+   {REMEMBER_BLOCK,UserRememberBlock},
 
 // Search
    {SEARCH_FORWARD,StartSearch},
@@ -122,14 +127,14 @@ ActionProcRec  EditorActionProcTable[]=
    {FORMAT_ONE_PARA,FormatPara},
    {FORMAT_DOCUMENT,FormatAll},
    {CENTER_LINE,CenterLine},
-   {AJUST_RIGHT_LINE,ShiftRightLine},
+   {ADJUST_RIGHT_LINE,ShiftRightLine},
    {FORMAT_FUNC_BAR,FormatFunc},
 
 // Others
    {CALCULATOR,editcalc},
    {DRAW_FRAMES,DrawFrames},
    {TABS_EXPAND,ExpandAllTabs},
-   {TEXT_OPTIMIZE,Optimize},
+   {TEXT_OPTIMIZE,UserOptimizeText},
    {CHOOSE_CHAR,UserChooseChar},
    {UNIX_DOS_TRANSFORM,DOS_UNIX},
 
@@ -173,6 +178,49 @@ ActionProcRec  EditorActionProcTable[]=
    {SWITCH_TEXT_MODE,UserSwitchTextMode},
    {SWITCH_GRAPH_MODE,UserSwitchGraphMode},
 
+   {EDIT_CHARSET,edit_chset},
+   {SET_CHARSET_8BIT,set_chset_8bit},
+   {SET_CHARSET_8BIT_NO_CONTROL,set_chset_8bit_noctrl},
+   {SAVE_TERMINAL_OPTIONS,SaveTermOpt},
+   {EDIT_COLORS,ColorsOpt},
+   {SAVE_COLORS,ColorsSave},
+   {SAVE_COLORS_FOR_TERM,ColorsSaveForTerminal},
+   {LOAD_COLOR_DEFAULT,LoadColorDefault},
+   {LOAD_COLOR_DEFBG,LoadColorDefaultBG},
+   {LOAD_COLOR_BLACK,LoadColorBlack},
+   {LOAD_COLOR_BLUE,LoadColorBlue},
+   {LOAD_COLOR_GREEN,LoadColorGreen},
+   {LOAD_COLOR_WHITE,LoadColorWhite},
+   {PROGRAMS_OPTIONS,ProgOpt},
+   {ABOUT,UserAbout},
+   {LOAD_KEYMAP_DEFAULT,LoadKeymapDefault},
+   {LOAD_KEYMAP_EMACS,  LoadKeymapEmacs},
+   {SAVE_KEYMAP,SaveKeymap},
+   {SAVE_KEYMAP_FOR_TERM,SaveKeymapForTerminal},
+
+   {SET_BOOKMARK,UserSetBookmark},
+   {SET_BOOKMARK_0,UserSetBookmark0},
+   {SET_BOOKMARK_1,UserSetBookmark1},
+   {SET_BOOKMARK_2,UserSetBookmark2},
+   {SET_BOOKMARK_3,UserSetBookmark3},
+   {SET_BOOKMARK_4,UserSetBookmark4},
+   {SET_BOOKMARK_5,UserSetBookmark5},
+   {SET_BOOKMARK_6,UserSetBookmark6},
+   {SET_BOOKMARK_7,UserSetBookmark7},
+   {SET_BOOKMARK_8,UserSetBookmark8},
+   {SET_BOOKMARK_9,UserSetBookmark9},
+   {GO_BOOKMARK,UserGoBookmark},
+   {GO_BOOKMARK_0,UserGoBookmark0},
+   {GO_BOOKMARK_1,UserGoBookmark1},
+   {GO_BOOKMARK_2,UserGoBookmark2},
+   {GO_BOOKMARK_3,UserGoBookmark3},
+   {GO_BOOKMARK_4,UserGoBookmark4},
+   {GO_BOOKMARK_5,UserGoBookmark5},
+   {GO_BOOKMARK_6,UserGoBookmark6},
+   {GO_BOOKMARK_7,UserGoBookmark7},
+   {GO_BOOKMARK_8,UserGoBookmark8},
+   {GO_BOOKMARK_9,UserGoBookmark9},
+
    {-1,NULL}
 };
 
@@ -206,8 +254,59 @@ void  EditorReadKeymap()
    if(errno)
    {
       FError(filename);
-      ActionCodeTable=DefaultActionCodeTable;
    }
 
    fclose(f);
 }
+
+void LoadKeymapEmacs()
+{
+   const char *k=PKGDATADIR"/keymap-emacs";
+   FILE *f=fopen(k,"r");
+   if(!f)
+   {
+      FError(k);
+      return;
+   }
+   ReadActionMap(f);
+   fclose(f);
+   RebuildKeyTree();
+   LoadMainMenu();
+}
+void LoadKeymapDefault()
+{
+   FreeActionCodeTable();
+   ActionCodeTable=DefaultActionCodeTable;
+   RebuildKeyTree();
+   LoadMainMenu();
+}
+void SaveKeymap()
+{
+   char  filename[1024];
+   FILE  *f;
+
+   sprintf(filename,"%s/.le/keymap",HOME);
+   f=fopen(filename,"w");
+   if(!f)
+   {
+      FError(filename);
+      return;
+   }
+   WriteActionMap(f);
+   fclose(f);
+}
+void SaveKeymapForTerminal()
+{
+   char  filename[1024];
+   FILE  *f;
+
+   sprintf(filename,"%s/.le/keymap-%s",HOME,TERM);
+   f=fopen(filename,"w");
+   if(!f)
+   {
+      FError(filename);
+      return;
+   }
+   WriteActionMap(f);
+   fclose(f);
+}
