@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1993-2000 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,12 +18,7 @@
 
 #include <config.h>
 #include <stdlib.h>
-#ifdef USE_NCURSES_H
-# include <ncurses.h>
-#else
-# include <curses.h>
-#endif
-
+#include "edit.h"
 #include "keynames.h"
 
 static const struct CodeName
