@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1993-2000 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: loadsave.cc,v 1.12 1998/11/26 15:49:02 lav Exp $ */
+/* $Id: loadsave.cc,v 1.18 2001/05/14 13:48:37 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -40,12 +40,13 @@
 #include "block.h"
 #include "clipbrd.h"
 #include "getch.h"
+#include "bm.h"
 
 #ifndef MAP_FAILED
 # define MAP_FAILED ((void*)-1)
 #endif
 
-#ifndef __MSDOS__
+#ifndef DISABLE_FILE_LOCKS
 int    LockFile(int fd,bool drop)
 {
    struct  flock   Lock;
@@ -89,7 +90,7 @@ int    LockFile(int fd,bool drop)
          case('I'):
             return(0);
          case('W'):
-            Message("Waiting for unlocking the file... (C-x - cancel)");
+            MessageSync("Waiting for unlocking the file... (C-x - cancel)");
             errno=EACCES;
             while(fcntl(fd,F_SETLK,&Lock)==-1 && (errno==EACCES || errno==EAGAIN))
             {
@@ -104,32 +105,27 @@ int    LockFile(int fd,bool drop)
                }
             }
             if(errno!=EACCES && errno!=EAGAIN)
-            {
-               FError(FileName);
                return(-2);
-            }
          }
       }
       else
       {
-         FError(FileName);
          return(-2);
       }
    }
    if(drop)
    {
       Lock.l_type=F_RDLCK;	// drop write lock to read lock
-      if(fcntl(fd,F_SETLK,&Lock)==-1)
-	 FError(FileName);
+      fcntl(fd,F_SETLK,&Lock);
    }
    return(0);
 }
-#else /* __MSDOS__ */
+#else /* DISABLE_FILE_LOCKS */
 int   LockFile(int,bool)
 {
    return 0;
 }
-#endif /* __MSDOS__ */
+#endif /* DISABLE_FILE_LOCKS */
 
 struct  menu   ConCan4Menu[]={
 {   " C&ontinue ",MIDDLE-6,FDOWN-2  },
@@ -177,10 +173,9 @@ off_t  GetDevSize(int fd)
 int   LoadFile(char *name)
 {
    struct stat    st;
-   register num   i;
+   offs   i;
    num    act_read;
    char   msg[256];
-   num    DosLastLine;
    InodeInfo   *old;
 
    CheckBlock();
@@ -188,6 +183,7 @@ int   LoadFile(char *name)
       MainClipBoard.Copy();
 
    EmptyText();
+   ResetBookmarks();
 
    flag=REDISPLAY_ALL;
 
@@ -201,7 +197,7 @@ int   LoadFile(char *name)
    }
 
    sprintf(msg,"Loading the file \"%.60s\"...",name);
-   Message(msg);
+   MessageSync(msg);
 
    newfile=0;
 
@@ -270,7 +266,7 @@ int   LoadFile(char *name)
 
    if(!buffer_mmapped)
    {
-      if(ReadBlock(file,st.st_size,&act_read)!=OK)
+      if(ReplaceTextFromFile(file,st.st_size,&act_read)!=OK)
       {
 	 if(errno)
 	    FError(name);
@@ -279,19 +275,21 @@ int   LoadFile(char *name)
       }
       CheckPoint();
 
-      DosLastLine=0;
-      for(i=Size()-1; i>0; )
+      num DosLastLine=0;
+      num UnixLastLine=0;
+      for(i=0; ; i++)
       {
-	 if(CharAt_NoCheck(--i)=='\r')
-	 {
-	    if(CharAt_NoCheck(i+1)=='\n')
-	       DosLastLine++;
-	 }
+	 i=ScanForCharForward(i,'\n');
+	 if(i==-1)
+	    break;
+	 UnixLastLine++;
+	 if(i>0 && CharAt_NoCheck(i-1)=='\r')
+	    DosLastLine++;
       }
-#ifndef __MSDOS__
-      if(TextEnd.Line()/2<DosLastLine) /* check if the file has unix or dos format */
+#if !defined(__MSDOS__) && !defined(__CYGWIN32__)
+      if(UnixLastLine/2<DosLastLine) /* check if the file has unix or dos format */
 #else
-      if(TextEnd.Line()/2<=DosLastLine)
+      if(UnixLastLine/2<=DosLastLine)
 #endif
       {
 	DosEol=1;
@@ -300,6 +298,10 @@ int   LoadFile(char *name)
 	TextPoint::OrFlags(COLUNDEFINED|LINEUNDEFINED);
 	TextEnd=TextPoint(Size(),DosLastLine,-1);
       }
+      else
+      {
+	 TextEnd=TextPoint(Size(),UnixLastLine,-1);
+      }
    }
    else /* buffer_mmapped */
    {
@@ -314,7 +316,7 @@ int   LoadFile(char *name)
       {
 	 buffer=(char*)mmap(0,st.st_size,PROT_READ|(View?0:PROT_WRITE),
 			    MAP_SHARED,file,0);
-	 if(buffer==MAP_FAILED)
+	 if(buffer==(char*)MAP_FAILED)
 	 {
 	    FError(name);
 	    EmptyText();
@@ -430,7 +432,7 @@ static int CreateBak(char *name)
       filename++;
    }
 
-   Message("Creating backup file...");
+   MessageSync("Creating backup file...");
 
    namemax=pathconf(directory,_PC_NAME_MAX);
    if(namemax==-1)
@@ -538,10 +540,10 @@ int   SaveFile(char *name)
    }
 
    if(Text && !View)
-      Optimize();
+      UserOptimizeText();
 
-   sprintf(msg,"Saving the file \"%s\"...",name);
-   Message(msg);
+   sprintf(msg,"Saving the file \"%.60s\"...",name);
+   MessageSync(msg);
 
    if(stat(name,&st)!=-1)
    {
@@ -608,7 +610,7 @@ int   SaveFile(char *name)
 
    newfile=0;
 
-   Message(msg);
+   MessageSync(msg);
 
    errno=0;
    nfile=open(name,O_CREAT|O_RDWR,st.st_mode);
@@ -628,10 +630,22 @@ int   SaveFile(char *name)
       ErrMsg("Warning: file locking failed");
 
    // now after locking truncate the file
+#ifdef HAVE_FTRUNCATE
+   ftruncate(nfile,0);
+#else
    close(open(name,O_TRUNC|O_RDONLY));
+#endif
 
-   /* force new file to be the same mode as source one */
-   chmod(name,st.st_mode);
+   struct stat new_st;
+   if(fstat(nfile,&new_st)!=-1 && new_st.st_mode!=st.st_mode)
+   {
+      /* force new file to be the same mode as source one */
+#ifdef HAVE_FCHMOD
+      fchmod(nfile,st.st_mode);
+#else
+      chmod(name,st.st_mode);
+#endif
+   }
 
    /* now, after all that stuff, write the buffer contents */
    errno=0;
