@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1993-2000 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,14 +25,15 @@
 #include "format.h"
 #include "colormnu.h"
 #include "about.h"
+#include "search.h"
 
-static bool RW() { return !View; }
-static bool BLK() { return !hide; }
-static bool noMM() { return !buffer_mmapped; }
-static bool RW_BLK() { return RW() && BLK(); }
-static bool RW_noMM() { return RW() && noMM(); }
-static bool RW_BLK_noMM() { return RW_BLK() && noMM(); }
-static bool RW_CLIP() { return RW() && !MainClipBoard.IsEmpty(); }
+#define RW  MENU_COND_RW
+#define BLK MENU_COND_BLOCK
+#define noMM MENU_COND_NO_MM
+#define RW_BLK MENU_COND_RW|MENU_COND_BLOCK
+#define RW_noMM MENU_COND_RW|MENU_COND_NO_MM
+#define RW_BLK_noMM MENU_COND_RW|MENU_COND_BLOCK|MENU_COND_NO_MM
+#define RW_CLIP MENU_COND_RW|MENU_COND_CLIPBOARD
 
 typedef void (*f)();
 
@@ -104,13 +105,13 @@ Menu1 MainMenu[]={
    {" format &All             ",FUNC+HIDE,FormatAll,	 RW_noMM  },
    {NULL},
 {" O&thers ",SUBM},
-   {" &Calculator         ",FUNC+HIDE,editcalc		       },
-   {" &Draw tables        ",FUNC+HIDE,DrawFrames,     RW_noMM  },
-   {" &Format functions   ",FUNC+HIDE,FormatFunc,     RW_noMM  },
-   {" &Expand all tabs    ",FUNC+HIDE,ExpandAllTabs,  RW_noMM  },
-   {" &Optimize text      ",FUNC+HIDE,Optimize,	      RW_noMM  },
-   {" character &Set      ",FUNC+HIDE,UserChooseChar	       },
-   {" &Unix<->Dos         ",FUNC+HIDE,DOS_UNIX,       RW_noMM  },
+   {" &Calculator         ",FUNC+HIDE,editcalc			  },
+   {" &Draw tables        ",FUNC+HIDE,DrawFrames,	 RW_noMM  },
+   {" &Format functions   ",FUNC+HIDE,FormatFunc,	 RW_noMM  },
+   {" &Expand all tabs    ",FUNC+HIDE,ExpandAllTabs,	 RW_noMM  },
+   {" &Optimize text      ",FUNC+HIDE,UserOptimizeText,	 RW_noMM  },
+   {" character &Set      ",FUNC+HIDE,UserChooseChar		  },
+   {" &Unix<->Dos         ",FUNC+HIDE,DOS_UNIX,		 RW_noMM  },
    {NULL},
 {" &Options ",SUBM},
    {" &Editor                      ",FUNC,Options},
