@@ -24,8 +24,13 @@ struct init
 };
 
 void SaveConfToOpenFile(FILE *f,const struct init *init);
+void SaveConfToFile(const char *f,const struct init *init);
+void ReadConfFromFile(const char *file,const struct init *init,bool mine);
 
 void ColorsOpt();
 void ProgOpt();
 
 extern const struct init colors[];
+
+extern bool ExplicitInitName;
+extern char InitName[256];
