@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1993-2000 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: screen.cc,v 1.23 1999/02/05 18:31:11 lav Exp $ */
+/* $Id: screen.cc,v 1.29 2000/09/04 08:19:02 lav Exp $ */
 
 #include <config.h>
 #include <string.h>
@@ -66,6 +66,10 @@ int le_mvaddchnstr(int line, int col, chtype *s, int n)
 
 void  TestPosition()
 {
+   int m=message_sp;
+   if(ShowStatusLine==SHOW_BOTTOM && m>0)
+      m--;  // one message is over status.
+
    if(hex)
    {
       if(ScreenTop&15)
@@ -82,18 +86,18 @@ void  TestPosition()
             ScreenTop=(Offset()&~15)-Scroll*16+16;
          }
       }
-      if(Offset()>=(ScreenTop.Offset()+16*TextWinHeight))
+      if(Offset()>=(ScreenTop.Offset()+16*(TextWinHeight-m)))
       {
          flag=REDISPLAY_ALL;
-         ScreenTop=(Offset()&~15)-(TextWinHeight-Scroll)*16;
+         ScreenTop=(Offset()&~15)-(TextWinHeight-m-Scroll)*16;
       }
       return;
    }
    num oldtop=ScreenTop.Line();
    num newtop;
-   if(GetLine()-oldtop>TextWinHeight-1)
+   if(GetLine()-oldtop>TextWinHeight-m-1)
    {
-      ScreenTop=PrevNLines(Offset(),TextWinHeight-Scroll);
+      ScreenTop=PrevNLines(Offset(),TextWinHeight-m-Scroll);
       newtop=ScreenTop.Line();
       flag=REDISPLAY_ALL;
    }
@@ -105,9 +109,11 @@ void  TestPosition()
    num   col=GetCol();
    if(Text && stdcol>col)
       col=stdcol;
-   if(ScrShift>col)
+   if(ScrShift+hscroll>col && ScrShift>0)
    {
-      ScrShift=(col/hscroll)*hscroll;
+      ScrShift=(col/hscroll-1)*hscroll;
+      if(ScrShift<0)
+	 ScrShift=0;
       flag=REDISPLAY_ALL;
    }
    else if(col-ScrShift>=TextWinWidth)
@@ -121,7 +127,11 @@ static int skipped=0;	// number of times Sync skipped its work
 
 void  SyncTextWin()
 {
-   int line=TextWinHeight;
+   int m=message_sp;
+   if(ShowStatusLine==SHOW_BOTTOM && m>0)
+      m--;  // one message is over status.
+
+   int line=TextWinHeight-m;
    int lim=-1;
    offs ptr;
 
@@ -154,12 +164,12 @@ void  SyncTextWin()
    if(flag&REDISPLAY_ALL)
    {
       line=0;
-      lim=TextWinHeight;
+      lim=TextWinHeight-m;
    }
    else if(flag&REDISPLAY_AFTER)
    {
       line=hex ? (CurrentPos-ScreenTop)/16-1 : GetLine()-ScreenTop.Line()-1;
-      lim=TextWinHeight;
+      lim=TextWinHeight-m;
    }
    else if(flag&REDISPLAY_LINE)
    {
@@ -190,6 +200,9 @@ void  SyncTextWin()
    else
       ptr=NextNLines(ScreenTop,line);
 
+   if(lim>TextWinHeight-m)
+      lim=TextWinHeight-m;
+
    if(lim>=line)
       Redisplay(line,ptr,lim);
    flag=0;
@@ -251,10 +264,9 @@ void  AddMessage(const char *s)
 {
    message_sp++;
    attrset(STATUS_LINE_ATTR->n_attr);
-   mvaddstr(LINES-message_sp,0,s);
+   mvaddstr(LINES-message_sp,0,(char*)s);
    for(int x=strlen(s); x<COLS; x++)
       addch(' ');
-   refresh();
 }
 void  Message(const char *s)
 {
@@ -264,13 +276,21 @@ void  Message(const char *s)
       ClearMessage();
    AddMessage(s);
 }
+void  MessageSync(const char *s)
+{
+   Message(s);
+   refresh();
+}
+
 void  ClearMessage()
 {
    if(message_sp>0 && (ShowStatusLine!=SHOW_BOTTOM || message_sp>1))
    {
-      Redisplay(TextWinHeight-message_sp,
-              hex?ScreenTop.Offset()+16*(TextWinHeight-message_sp)
-                :NextNLines(ScreenTop.Offset(),TextWinHeight-message_sp),
+      int m=message_sp;
+      message_sp=0;
+      Redisplay(TextWinHeight-m,
+              hex?ScreenTop.Offset()+16*(TextWinHeight-m)
+                :NextNLines(ScreenTop.Offset(),TextWinHeight-m),
               TextWinHeight);
       ScrollBar(false);
    }
@@ -391,6 +411,10 @@ void  Redisplay(num line,offs ptr,num limit)
    char  s[64],*sp;
    offs  lptr;
 
+   int m=message_sp;
+   if(ShowStatusLine==SHOW_BOTTOM && m>0)
+      m--;  // one message is over status.
+
    norm_attr=NORMAL_TEXT_ATTR;
    blk_attr=BLOCK_TEXT_ATTR;
    syntax[0]=find_attr(SYNTAX1);
@@ -407,15 +431,15 @@ void  Redisplay(num line,offs ptr,num limit)
       ptr=ScreenTop.Offset();
       line=0;
    }
-   if(limit>TextWinHeight)
-      limit=TextWinHeight;
+   if(limit>TextWinHeight-m)
+      limit=TextWinHeight-m;
 
    if(flag&REDISPLAY_ALL)
    {
       ScrollBar(FALSE);	/* redraw all the scrollbar */
       line=0;
       ptr=ScreenTop.Offset();
-      limit=TextWinHeight;
+      limit=TextWinHeight-m;
    }
 
    if(limit<=line)
@@ -454,10 +478,11 @@ void  Redisplay(num line,offs ptr,num limit)
 	    *clp++=ca->n_attr|s[0];
 	    *clp++=ca->n_attr|s[1];
             ptr++;
-            if(InBlock(ptr-1) && InBlock(ptr) && (ptr&15))
-               *clp++=blk_attr->n_attr|' ';
+	    char b=' '; // ((ptr&15)!=8 ? ' ' : '-');
+	    if(InBlock(ptr-1) && InBlock(ptr) && (ptr&15))
+               *clp++=blk_attr->n_attr|b;
             else
-               *clp++=norm_attr->n_attr|' ';
+               *clp++=norm_attr->n_attr|b;
          }
 	 while(clp-cl<AsciiPos)
 	    *clp++=norm_attr->n_attr|' ';
@@ -489,7 +514,7 @@ void  Redisplay(num line,offs ptr,num limit)
       if(hl_option && hl_active)
       {
 	 start=PrevNLines(ScreenTop,hl_lines-1);
-	 end=NextNLines(ScreenTop,TextWinHeight+hl_lines);
+	 end=NextNLines(ScreenTop,TextWinHeight-m+hl_lines);
 	 int ll=end-start;
 	 if(ll==0)
 	    goto after_hl;
@@ -527,7 +552,7 @@ void  Redisplay(num line,offs ptr,num limit)
 	 offs p=ScreenTop;
 	 hlp=hl+(p-start);
 	 int l;
-	 for(l=0; l<TextWinHeight && l<1024; l++)
+	 for(l=0; l<TextWinHeight-m && l<1024; l++)
 	 {
 	    next_line_ptr=NextLine(p);
 	    newhash=mkhash(hlp,next_line_ptr-p);
@@ -551,13 +576,28 @@ void  Redisplay(num line,offs ptr,num limit)
       }
    after_hl:
 
-      for(; line<limit; line++,ptr=next_line_ptr)
+      for(;;)
       {
-	 next_line_ptr=NextLine(ptr);
+	 col=ScrShift-TabSize+1;
+	 if(col<0)
+	    col=0;
+
+	 TextPoint n(ScreenTop.Line()+line,col);
+	 next_line_ptr=n.Offset();
+
+	 if(hlp)
+	    hlp+=next_line_ptr-ptr;
+
+	 ptr=next_line_ptr;
+
+	 if(n.Line()!=ScreenTop.Line()+line)
+	    col=0;
+	 else
+	    col=n.Col()-ScrShift;
+
 	 clp=cl;
 
-         for(col=(-ScrShift);
-            col<TextWinWidth && !EolAt(ptr); ptr++)
+         for( ; col<TextWinWidth && !EolAt(ptr); ptr++)
          {
 	    if(col>-TabSize)
 	       ca=FindPosAttr(ptr,line,col,hlp);
@@ -568,6 +608,8 @@ void  Redisplay(num line,offs ptr,num limit)
                i=Tabulate(col+ScrShift)-ScrShift;
                if(i>0)
 	       {
+		  if(col<0)
+		     col=0;
 		  *clp++ = ca->n_attr|' ';
 		  col++;
 		  while(col<i && col<TextWinWidth)
@@ -604,9 +646,11 @@ void  Redisplay(num line,offs ptr,num limit)
 	 attrset(0);
 	 mvaddchnstr(TextWinY+line,TextWinX,cl,TextWinWidth);
 
-	 if(hlp)
-	    hlp+=next_line_ptr-ptr;
-      }
+	 if(++line>=limit)
+	    break;
+
+      } // end for(;;)
+
       if(hl)
 	 free(hl);
    }
@@ -614,25 +658,13 @@ void  Redisplay(num line,offs ptr,num limit)
 
 void  RedisplayAll()
 {
-   ScrollBar(FALSE);
-   flag=0;
-   Redisplay(0,ScreenTop.Offset(),TextWinHeight);
-}
-void  RedisplayAfter()
-{
-   flag&=~REDISPLAY_AFTER;
-   if(hex)
-      Redisplay((Offset()-ScreenTop.Offset())/16-1,(Offset()&~15)-16,TextWinHeight);
-   else
-      Redisplay(GetLine()-ScreenTop.Line()-1,PrevLine(Offset()),TextWinHeight);
+   flag=REDISPLAY_ALL;
+   SyncTextWin();
 }
 void  RedisplayLine()
 {
-   flag&=~REDISPLAY_LINE;
-   if(hex)
-      Redisplay((Offset()-ScreenTop.Offset())/16-1,(Offset()&~15)-16,(Offset()-ScrPtr)/16+2);
-   else
-      Redisplay(GetLine()-ScreenTop.Line()-1,PrevLine(Offset()),GetLine()-ScrLine+1);
+   flag|=REDISPLAY_LINE;
+   SyncTextWin();
 }
 
 void  CenterView()
@@ -666,7 +698,10 @@ void  CenterView()
    }
    offs max_top=PrevNLines(TextEnd,TextWinHeight-1);
    if(max_top<ScreenTop)
+   {
       ScreenTop=max_top;
+      flag=REDISPLAY_ALL;
+   }
 }
 
 struct  menu   OkMenu[]={
