@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1993-2000 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  */
 
 /* Search & replace for the editor */
-/* $Id: search.cc,v 1.26 1999/11/06 16:57:54 lav Exp $ */
+/* $Id: search.cc,v 1.33 2001/03/16 13:27:35 lav Exp $ */
 
 #include <config.h>
 #include <memory.h>
@@ -27,12 +27,15 @@
 #include "edit.h"
 #include "keymap.h"
 #include "getch.h"
+#include "search.h"
+#include "xalloca.h"
 
 extern "C" {
 #ifdef WITH_REGEX
    #include <regex.h>
 #else
    #include <rx.h>
+   #define RE_TRANSLATE_TYPE unsigned char *
 #endif
 }
 
@@ -56,7 +59,6 @@ num   fndlen=0;
 static struct re_registers regs;
 static struct re_pattern_buffer rexp;
 static bool rexp_compiled=false;
-static char fastmap[256];
 static bool word_bounds;
 static bool numeric_search;
 static bool hex_search;
@@ -79,11 +81,6 @@ char *my_memrchr(const char *mem,char ch,int len)
 
 void  NotFound()
 {
-   if(ShowStatusLine!=SHOW_BOTTOM && GetLine()-ScreenTop.Line()==TextWinHeight-1)
-   {
-      ScreenTop=NextLine(ScreenTop);
-      flag=REDISPLAY_ALL;
-   }
    CurrentPos=back_tp;
    SyncTextWin();
    StatusLine();
@@ -93,8 +90,8 @@ void  NotFound()
    WaitForKey();
 }
 
-static unsigned char map_to_lower[256];
-static void map_to_lower_init()
+unsigned char map_to_lower[256];
+void map_to_lower_init()
 {
    if(map_to_lower[' '])  // assumes tolower(' ')!='\0' :-)
       return;
@@ -110,8 +107,17 @@ static bool CompilePattern()
    if(noreg)
       return true;
 
-//    if(rexp_compiled)
-//       regfree(&rexp);
+   if(rexp_compiled)
+   {
+      regfree(&rexp);
+      rexp_compiled=false;
+      memset(&rexp,0,sizeof(rexp));
+      if(regs.start)
+	 free(regs.start);
+      if(regs.end)
+	 free(regs.end);
+      memset(&regs,0,sizeof(regs));
+   }
 
    re_syntax_options=RE_BK_PLUS_QM|RE_CHAR_CLASSES|RE_CONTEXT_INDEP_ANCHORS|
 		     RE_UNMATCHED_RIGHT_PAREN_ORD;
@@ -131,7 +137,8 @@ static bool CompilePattern()
 	 {
 	    case('i'): // ignore case
 	       map_to_lower_init();
-	       rexp.translate=map_to_lower;
+	       rexp.translate=(RE_TRANSLATE_TYPE)malloc(256);
+	       memcpy(rexp.translate,map_to_lower,256);
 	       break;
 	    case('w'):
 	       word_bounds=true;
@@ -183,7 +190,8 @@ static bool CompilePattern()
       return false;
    }
    rexp_compiled=true;
-   rexp.fastmap=fastmap;
+   rexp.fastmap=(char*)malloc(256);
+   re_compile_fastmap(&rexp);
    return true;
 }
 
@@ -300,7 +308,7 @@ int   no_re_search_2(const char *str,const int slen,
 
 int    Search(int dir,offs offslim)
 {
-   Message(dir==FORWARD?"Searching forwards...":"Searching backwards...");
+   MessageSync(dir==FORWARD?"Searching forwards...":"Searching backwards...");
 
    offs srchpos=CurrentPos;
 
@@ -344,6 +352,11 @@ search_again:
    }
    if(res==-1)
       return FALSE;
+   if(res==-2)
+   {
+      ErrMsg("re_search_2 failed");
+      return FALSE;
+   }
 
    fndind=res;
    if(noreg)
@@ -398,7 +411,7 @@ void  ReplaceFound()
 	    scan+=len;
 	    scan_len-=len;
 	 }
-	 if(!bslash)
+	 if(!bslash || scan_len==0)
 	    break;
 	 scan++;
 	 scan_len--;
@@ -519,13 +532,15 @@ void  Replace()
       flag=REDISPLAY_ALL;
       if(key!='*' && key!='#')
       {
-         int    oldr=rblock;
-         TextPoint   OldBlockBegin=BlockBegin,OldBlockEnd=BlockEnd;
-         int    oldh=hide;
+         int	     oldr=rblock;
+         TextPoint   OldBlockBegin=BlockBegin;
+	 TextPoint   OldBlockEnd  =BlockEnd;
+         int	     oldh=hide;
          BlockBegin=BlockEnd=CurrentPos;
          BlockEnd+=fndlen;
          hide=0;
          rblock=0;
+	 stdcol=GetCol();
          CenterView();
          SyncTextWin();
          StatusLine();
@@ -791,7 +806,7 @@ void  FindMatch()
             level--;
       }
    }
-   Message("Matching bracket not found");
+   Message("Matching bracket not found.");
    SetCursor();
    WaitForKey();
 }
