@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1993-2000 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: user.cc,v 1.21 1999/11/06 16:57:54 lav Exp $ */
+/* $Id: user.cc,v 1.34 2001/05/22 10:58:24 lav Exp $ */
 
 #include <config.h>
 #include <ctype.h>
@@ -35,6 +35,7 @@
 #include "getch.h"
 #include "format.h"
 #include "about.h"
+#include "bm.h"
 
 void  UserDeleteToEol()
 {
@@ -72,7 +73,24 @@ void  UserLineDown()
    }
    else
    {
+      num line=GetLine();
       MoveDown();
+      if(line>=GetLine())
+      {
+	 if(Text)
+	 {
+	    if(!Bol())
+	    {
+	       num old_stdcol=stdcol;
+	       int old_modified=modified;
+	       InsertChar('\n');
+	       stdcol=old_stdcol;
+	       modified=old_modified;
+	    }
+	 }
+	 else
+	    stdcol=GetCol();
+      }
    }
 }
 
@@ -208,6 +226,9 @@ void  UserCopyFromUp()
 
 void  UserDeleteBlock()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    if(View)
       return;
    CheckBlock();
@@ -219,6 +240,9 @@ void  UserDeleteBlock()
 }
 void  UserCopyBlock()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    if(View)
       return;
    CheckBlock();
@@ -231,6 +255,9 @@ void  UserCopyBlock()
 }
 void  UserMoveBlock()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    if(View)
        return;
    CheckBlock();
@@ -314,6 +341,9 @@ void  UserDeleteWord()
 
 void  UserMarkWord()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    offs word_begin=CurrentPos;
    offs word_end=CurrentPos;
    while(!BofAt(word_begin) && isalnum(CharAt(word_begin-1)))
@@ -329,6 +359,9 @@ void  UserMarkWord()
 }
 void  UserMarkLine()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    BlockBegin=LineBegin(Offset());
    if(rblock)
       BlockEnd=BlockBegin;
@@ -339,6 +372,9 @@ void  UserMarkLine()
 }
 void  UserMarkToEol()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    stdcol=GetCol();
    BlockBegin=CurrentPos;
    BlockEnd=LineEnd(CurrentPos.Offset());
@@ -347,6 +383,9 @@ void  UserMarkToEol()
 }
 void  UserMarkAll()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    BlockBegin=TextBegin;
    if(rblock)
       BlockEnd=LineBegin(TextEnd);
@@ -1004,7 +1043,7 @@ void  UserToLineNumber()
 }
 void  UserToOffset()
 {
-   static char no[10]="";
+   static char no[40]="";
    if(getstring("Move to offset: ",no,sizeof(no)-1,NULL,NULL,NULL)<1)
       return;
    CurrentPos=strtol(no,0,0);
@@ -1348,6 +1387,12 @@ void  UserSwitchAutoindentMode()
 
 void  UserBlockPrefixIndent()
 {
+   if(View)
+      return;
+
+   if(DragMark)
+      UserStopDragMark();
+
    static char str[256];
    static int  len=0;
 
@@ -1369,6 +1414,9 @@ void  UserShellCommand()
 
 void  UserPipeBlock()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    static char filter[256];
 
    CheckBlock();
@@ -1378,14 +1426,17 @@ void  UserPipeBlock()
    if(getstring("Pipe through: ",filter,sizeof(filter)-1,NULL,NULL,NULL)<1)
       return;
 
-   Message("Piping...");
+   MessageSync("Piping...");
 
    PipeBlock(filter,TRUE,TRUE);
-
+   flag=REDISPLAY_ALL;
 }
 
 void  UserYankBlock()
 {
+   if(DragMark)
+      UserStopDragMark();
+
    if(View)
       return;
    MainClipBoard.PasteAndMark();
@@ -1489,8 +1540,7 @@ static void post_mark_move()
    }
    else
    {
-      if(*mark_move_point!=CurrentPos)
-	 flag=REDISPLAY_ALL;
+      flag=REDISPLAY_ALL;
       *mark_move_point=CurrentPos;
       if(BlockBegin>BlockEnd)
       {
@@ -1506,7 +1556,9 @@ static void post_mark_move()
    void UserMark##move()   \
    {			   \
       pre_mark_move();	   \
+      hide=1;		   \
       User##move();	   \
+      SeekStdCol();	   \
       post_mark_move();	   \
    }
 MarkMove(CharLeft);
@@ -1523,3 +1575,72 @@ MarkMove(PageTop);
 MarkMove(PageBottom);
 MarkMove(LineUp);
 MarkMove(LineDown);
+
+
+void UserOptimizeText()
+{
+   if(View || buffer_mmapped)
+      return;
+
+   offs     ptr;
+   TextPoint  tp=CurrentPos;
+
+   MessageSync("Optimizing...");
+   for(ptr=0; !EofAt(ptr); ptr++)
+   {
+      if(EolAt(ptr))
+      {
+         CurrentPos=ptr;
+         while(!Bol() && (CharRel_NoCheck(-1)==' ' || CharRel_NoCheck(-1)=='\t'))
+            BackSpace();
+      }
+   }
+   CurrentPos=TextEnd;
+   while(!Bof() && Bol() && BolAt(Offset()-EolSize))
+      DeleteBlock(EolSize,0);
+   if(!Bol())
+      NewLine();
+
+   CurrentPos=tp;
+   stdcol=GetCol();
+   flag=REDISPLAY_ALL;
+}
+
+void UserRememberBlock()
+{
+   if(DragMark)
+      UserStopDragMark();
+
+   MainClipBoard.Copy();
+}
+
+void UserSetBookmark()
+{
+   Message("Mark: ");
+   move(LINES-1,6);
+   curs_set(1);
+   int key=getch();
+   if(key<256 && key>=0)
+      SetBookmark(key);
+   else
+      beep();
+   ClearMessage();
+}
+
+void UserGoBookmark()
+{
+   Message("Go to mark: ");
+   move(LINES-1,12);
+   curs_set(1);
+   int key=getch();
+   if(key<256 && key>=0)
+      GoBookmark(key);
+   else
+      beep();
+   ClearMessage();
+}
+
+#define S(n) void UserSetBookmark##n() { SetBookmark('0'+n); }
+S(0) S(1) S(2) S(3) S(4) S(5) S(6) S(7) S(8) S(9)
+#define G(n) void UserGoBookmark##n() { GoBookmark('0'+n); }
+G(0) G(1) G(2) G(3) G(4) G(5) G(6) G(7) G(8) G(9)
